************************************************************************
PROCEDURE GGRAPH
******************
***    Author: (c) Rick Strahl, 1994
***  Modified: 04/24/94
***  Function: This function works as a front end to GENGRAPH to allow
***            simple, automatic graph generation without the pesky
***            Graph Wizard and the complexity of using UPDGRAPH.
***            Graph type and display mode can be controlled via
***            the parameters. The graph types supported are
***            VBAR,HBAR,PIE,AREA and LINE in both 2D and 3D
***            flavors. You can add specialty graphs to GRAPHS.DBF
***            by creating the graph and changing the graphname
***            field string to a keyword of your choice which can
***            then be used as the pcGraphType parameter. For
***            example you might want to add Scatter charts or a
***            special Stacked or Bar Chart.
***
***            Output can be sent to an OLE editable 'MEMO' display
***            (like GENGRAPH does), the GGRAPH 'WINDOW' with buttons
***            for all graphtypes and a print option, and to a 'CURSOR'.
***
***            If sending output to a cursor, a name must be supplied.
***            The first pass through GGRAPH will create the cursor
***            for you, naming it to your parameter or TMPGRAPH by
***            default if no parameter is passed. When GGRAPH returns
***            your cursor will contain a graph record with a general
***            field (OLEGRAPH). Subsequent calls with this cursor
***            to GGRAPH will add append the graphs to it so you can
***            collect a group of graphs and display them as general
***            fields later under your control.
***            Your code is responsible for closing the cursor...
***
***            This routine makes use of a table called GRAPHS that
***            contains the templates for the graph types supported
***            here. The GraphType is translated to a record number
***            in this template table. You can change the default
***            behavior of the graphs in the GRAPHS table by browsing
***            and selecting the OLEGRAPH field and double clicking
***            while the edit window is active, which brings forth
***            MS GRAPH.
***
***
***    Assume: GENGRAPH.APP is available via the _GENGRAPH variable.
***            Same assumptions as GENGRAPH...
***            The data to be graphed must be open and selected.
***            The first field in the table must be character.
***            Any numeric values in the table are graphed.
***            Preferably call GGRAPH only with the data you need
***            displayed in the graph.
***
***            calls GGRAPH.SPR for 'WINDOW' mode interface.
***
***
***      pass: pcGraphType   -   *"HBAR","VBAR","PIE","LINE","AREA"
***                              "3D" maybe appended to each above
***            pcMethod      -   *"MEMO","WINDOW","CURSOR"
***            pcTitle       -   Title displayed ontop the graph
***            pcWinTitle    -   Window Title (defaults to pcTitle)
***            pcCursor      -   Name of the Cursor if CURSOR is used.
***                              otherwise this parm is ignored
***                              (defaults to TMPGRAPH)
***
***            ALL PARAMETERS ARE OPTIONAL... (* signify defaults)
***
***  Examples: do GGRAPH with "VBAR3D","MEMO"," Monthly Sales Totals ",;
***                           "Invoice Sales"
***
***            do GGRAPH with "PIE3D","WINDOW"," Monthly Sales Totals ",;
***                           " Invoice Sales "
***
***            do GGRAPH with "LINE3D","CURSOR"," Monthly Sales Totals ",;
***                           " Invoice Sales ","TGRAPHS"
***
***
***    Return: .f. on failure  .t. on success
*************************************************************************
PARAMETER m.pcGraphType,m.pcMethod,m.pcTitle,m.pcWinTitle,m.pcCursor

PRIVATE m.ll3D,m.lcOldAlias,m.lcOldSafety,m.lnGraphRec,m.lcTempFile

*** _MAC will work OK, but GGRAPH.SCX
*** must be adjusted to scale properly
IF _DOS
   wait window "This program requires FoxPro Windows..." NOWAIT
   RETURN .F.
ENDIF

*** Make sure _GENGRAPH points at GENGRAPH.APP
if !file(_GENGRAPH)
   ?? chr(7)
   wait window TIMEOUT 3 ;
        "Unable to find GENGRAPH.APP!"+chr(13)+;
        "Make sure the _GENGRAPH system variable is set..."
   RETURN .f.    
endif

*** Make sure table is open
IF EMPTY(ALIAS())  or (ALIAS()="GRAPHS")
   WAIT WINDOW TIMEOUT 3 ;
      "GGRAPH.APP - No datafile passed... Returning to caller"
   RETURN .F.
ENDIF

m.pcGraphType=IIF(TYPE("pcGraphType")="C",UPPER(TRIM(m.pcGraphType)),"HBAR")
m.pcTitle=IIF(TYPE("PCTITLE")="C",m.pcTitle,"")
m.pcMethod=IIF(TYPE("PCMETHOD")="C",UPPER(m.pcMethod),"MEMO")
m.pcWinTitle=IIF(TYPE("PCWINTITLE")="C",m.pcWinTitle,m.pcTitle)
m.pcCursor=IIF(TYPE("PCCURSOR")="C",m.pcCursor,"TMPGRAPH")

*** Setup functions
m.lcOldAlias=ALIAS()
m.lcOldSafety=SET("SAFETY")
SET SAFETY OFF

m.ll3D=.F.   && Flag the 3D option chkbox for the Window

*** Strip off 3D to find out if 3D graph
IF RIGHT(m.pcGraphType,2)="3D"
   m.ll3D=.T.
ENDIF

*** Temp File Name - Dump OLE into this file
m.lcTempFile=IIF(!EMPTY(GETENV("TEMP")),GETENV("TEMP")+"\"+SYS(3),SYS(3))

*** Create a graph cursor - Alias GGRAPH
DO mk_graph WITH m.pcGraphType,m.pcTitle,m.pcWinTitle

*** Display an OLE editable memo
IF m.pcMethod="MEMO"
   DEFINE WINDOW wdmgraph ;
      AT  0.000, 0.000  ;
      SIZE 25.067,99.167 ;
      TITLE m.pcWinTitle ;
      FONT "Arial", 9 STYLE 'B';
      FLOAT ;
      CLOSE ;
      MINIMIZE ;
      GROW ;
      COLOR RGB(,,,192,192,192)
   MOVE WINDOW wdmgraph CENTER

   WAIT WINDOW NOWAIT ;
       "Press ESC to exit - ENTER or double click to bring up MS GRAPH..."
   MODI GENERAL ggraph.olegraph WINDOW wdmgraph
   RELEASE WINDOW wdmgraph
   WAIT CLEAR
ENDIF

*** Call the GGRAPH Screen
IF m.pcMethod="WINDOW"
   *** Save old Font Settings settings so window sizes properly
   PRIVATE m.lcFont_name,m.lnFont_size,m.lcFont_style

   m.lcfont_name=WFONT(1,"")
   m.lnfont_size=WFONT(2,"")
   m.lcfont_style=WFONT(3,"")

   MODI WINDOW SCREEN FONT "Arial",9 STYLE 'N'

   DO ggraph.spr

   MODI WINDOW SCREEN FONT (m.lcfont_name),(m.lnfont_size) STYLE (m.lcfont_style)
ENDIF

*** Close the template graph database
SELE ggraph
USE

*** Create or Append to Cursor
IF m.pcMethod="CURSOR"
   IF !USED(m.pcCursor)
      CREATE CURSOR (m.pcCursor) ;
         (   GRAPHNAME C ( 30),;
         GRAPHID N (  2,  0),;
         GRAPHTITLE C ( 80),;
         XAXIS C ( 12),;
         FLIST M ,;
         OLEGRAPH G )
   ENDIF

   SELE (m.pcCursor)
   APPEND FROM (m.lcTempFile)
ENDIF

*** Erase the temp template file
ERASE (m.lcTempFile+".DBF")
ERASE (m.lcTempFile+".FPT")

IF m.lcOldSafety="ON"
   SET SAFETY ON
ENDIF

SELE (m.lcOldAlias)

RETURN .T.
* EOP GGRAPH

**********************************************************************
PROCEDURE MK_GRAPH
******************
*  This is the workhorse routine of GGRAPH. It creates a
*  temporary Graph Database with one record from the
*  template DBF and uses UPDGRAPH to modify the graph
*  using the current datafile as input. GGRAPH is then
*  passed back to the calling program to process there.
**********************************************************************
PARAMETER m.pcGraphType,m.pcTitle,m.pcWinTitle

PRIVATE A_FIELDS,A_TFIELDS,m.lcXaxis

*** Dump the fields into an array
fld_count=AFIELDS(a_tfields)
IF fld_count<1 OR EMPTY(m.lcOldAlias)
   WAIT WINDOW "Unable to graph datafile..."
   RETURN
ENDIF

WAIT WINDOW "Generating Graph..." NOWAIT

*** Create a 1D array of the fields and 
*** pass to UPDATEGRPH
DIMENSION a_fields(fld_count)
FOR x=1 TO fld_count
   a_fields[x]=a_tfields[x,1]
ENDFOR

*** Title Column is the first field
m.lcXaxis=a_fields[1]

*** Open Graph Template File
IF !USED("GRAPHS")
   USE GRAPHS IN 0
ENDIF

*** Now find the graph in the template file
SELE graphs
LOCATE FOR upper(trim(graphs.graphname))==pcGraphType

*** Check for invalid entry. Use HBAR3D if no match
IF !FOUND()
   LOCATE FOR graphs.graphname="HBAR3D"
ENDIF

*** Copy the selected Graph to temp file
COPY NEXT 1 TO &lcTempFile
USE

*** Re-select the alias to graph
SELE (m.lcOldAlias)

DO ("UPDATEGRPH") IN (_GENGRAPH) WITH ;
   (m.lcTempFile+".DBF"),1,2,;
   m.pcTitle,a_fields,m.lcXaxis

*** In case temp file is open - 
*** close it ('WINDOW' mode cycling)
IF USED("GGRAPH")
   SELE GGRAPH
   USE
ENDIF

*** Now open the newly created file and return
USE (m.lcTempFile) IN 0 ALIAS GGRAPH

WAIT CLEAR

RETURN
* EOP MK_GRAPH

